@@ -1,8 +1,13 @@
 /**************************************************************************
- Copyright:
-      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
- License: See LICENSE_gsmartcontrol.txt
+Copyright:
+	(C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
+License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup applib
+/// \weakgroup applib
+/// @{
 
 #include "rconfig/rconfig_mini.h"
 #include "hz/string_algo.h"  // string_trim_copy, string_any_to_unix_copy
@@ -21,43 +26,45 @@
 
 std::string StorageDevice::get_type_readable_name(StorageDevice::detected_type_t type)
 {
-    switch (type) {
-    case detected_type_unknown:
-        return "unknown";
-    case detected_type_invalid:
-        return "invalid";
-    case detected_type_cddvd:
-        return "cd/dvd";
-    }
-    return "[internal_error]";
+	switch (type) {
+	case detected_type_unknown:
+		return "unknown";
+	case detected_type_invalid:
+		return "invalid";
+	case detected_type_cddvd:
+		return "cd/dvd";
+	case detected_type_raid:
+		return "raid";
+	}
+	return "[internal_error]";
 }
 
 
 
 std::string StorageDevice::get_status_name(StorageDevice::status_t status, bool use_yesno)
 {
-    switch (status) {
-    case status_enabled:
-        return (use_yesno ? "Yes" : "Enabled");
-    case status_disabled:
-        return (use_yesno ? "No" : "Disabled");
-    case status_unsupported:
-        return "Unsupported";
-    case status_unknown:
-        return "Unknown";
-    };
-    return "[internal_error]";
+	switch (status) {
+	case status_enabled:
+		return (use_yesno ? "Yes" : "Enabled");
+	case status_disabled:
+		return (use_yesno ? "No" : "Disabled");
+	case status_unsupported:
+		return "Unsupported";
+	case status_unknown:
+		return "Unknown";
+	};
+	return "[internal_error]";
 }
 
 
 
-StorageDevice::StorageDevice(const string& dev_or_vfile, bool is_virtual)
+StorageDevice::StorageDevice(const std::string& dev_or_vfile, bool is_virtual)
 {
 	detected_type_ = detected_type_unknown;
 	// force_type_ = false;
 	is_virtual_ = is_virtual;
 	is_manually_added_ = false;
-	fully_parsed_ = false;
+	parse_status_ = parse_status_none;
 	test_is_active_ = false;
 
 	if (is_virtual) {
@@ -69,13 +76,13 @@ StorageDevice::StorageDevice(const string& dev_or_vfile, bool is_virtual)
 
 
 
-StorageDevice::StorageDevice(const string& dev, const string& type_arg)
+StorageDevice::StorageDevice(const std::string& dev, const std::string& type_arg)
 {
 	detected_type_ = detected_type_unknown;
 	// force_type_ = false;
 	is_virtual_ = false;
 	is_manually_added_ = false;
-	fully_parsed_ = false;
+	parse_status_ = parse_status_none;
 	test_is_active_ = false;
 
 	device_ = dev;
@@ -86,62 +93,62 @@ StorageDevice::StorageDevice(const string& dev, const string& type_arg)
 
 StorageDevice::StorageDevice(const StorageDevice& other)
 {
-    *this = other;
+	*this = other;
 }
 
 
 
 StorageDevice& StorageDevice::operator=(const StorageDevice& other)
 {
-    info_output_ = other.info_output_;
-    full_output_ = other.full_output_;
+	info_output_ = other.info_output_;
+	full_output_ = other.full_output_;
 
-    device_ = other.device_;
+	device_ = other.device_;
 	type_arg_ = other.type_arg_;
 	extra_args_ = other.extra_args_;
 
-    // force_type_ = other.force_type_;
-    is_virtual_ = other.is_virtual_;
-    virtual_file_ = other.virtual_file_;
-    is_manually_added_ = other.is_manually_added_;
+	// force_type_ = other.force_type_;
+	is_virtual_ = other.is_virtual_;
+	virtual_file_ = other.virtual_file_;
+	is_manually_added_ = other.is_manually_added_;
 
-    fully_parsed_ = other.fully_parsed_;
-    test_is_active_ = other.test_is_active_;
+	parse_status_ = other.parse_status_;
+	test_is_active_ = other.test_is_active_;
 
-    detected_type_ = other.detected_type_;
-    smart_supported_ = other.smart_supported_;
-    smart_enabled_ = other.smart_enabled_;
-    aodc_status_ = other.aodc_status_;
-    model_name_ = other.model_name_;
-    family_name_ = other.family_name_;
-    size_ = other.size_;
-    health_property_ = other.health_property_;
+	detected_type_ = other.detected_type_;
+	smart_supported_ = other.smart_supported_;
+	smart_enabled_ = other.smart_enabled_;
+	aodc_status_ = other.aodc_status_;
+	model_name_ = other.model_name_;
+	family_name_ = other.family_name_;
+	size_ = other.size_;
+	health_property_ = other.health_property_;
 
-    properties_ = other.properties_;
+	properties_ = other.properties_;
 
-    return *this;
+	return *this;
 }
 
 
 
 void StorageDevice::clear_fetched(bool including_outputs) {
-    if (including_outputs) {
-        info_output_.clear();
-        full_output_.clear();
-    }
+	if (including_outputs) {
+		info_output_.clear();
+		full_output_.clear();
+	}
 
-    fully_parsed_ = false;
-    test_is_active_ = false;  // not sure
+	parse_status_ = parse_status_none;
+	test_is_active_ = false;  // not sure
 
-    smart_supported_.reset();
-    smart_enabled_.reset();
-    model_name_.reset();
-    aodc_status_.reset();
-    family_name_.reset();
-    size_.reset();
-    health_property_.reset();
+	smart_supported_.reset();
+	smart_enabled_.reset();
+	model_name_.reset();
+	aodc_status_.reset();
+	family_name_.reset();
+	size_.reset();
+	health_property_.reset();
 
-    properties_.clear();
+	properties_.clear();
 }
 
 
@@ -196,35 +203,53 @@ std::string StorageDevice::parse_basic_data(bool do_set_properties, bool emit_si
 	if (!SmartctlParser::parse_version(this->info_output_, version, version_full))  // is this smartctl data at all?
 		return "Cannot get smartctl version information.";
 
-	// detect type. note: we can't distinguish between sata and scsi (on linux, for -d ata switch).
-	if (app_pcre_match("/this device: CD\\/DVD/mi", info_output_)) {
+	// Detect type. note: we can't distinguish between sata and scsi (on linux, for -d ata switch).
+	// Sample output line 1 (encountered on a CDRW drive):
+	// SMART support is: Unavailable - Packet Interface Devices [this device: CD/DVD] don't support ATA SMART
+	// Sample output line 2 (encountered on a BDRW drive):
+	// Device type:          CD/DVD
+	if (app_pcre_match("/this device: CD\\/DVD/mi", info_output_) || app_pcre_match("/^Device type:\\s+CD\\/DVD/mi", info_output_)) {
+		debug_out_dump("app", "Drive " << get_device_with_type() << " seems to be a CD/DVD device.\n");
 		this->set_detected_type(detected_type_cddvd);
+
+	// This was encountered on a csmi soft-raid under windows with pd0.
+	// The device reported that it had smart supported and enabled.
+	// Product:              Raid 5 Volume
+	} else if (app_pcre_match("/Product:[ \\t]*Raid/mi", info_output_)) {
+		debug_out_dump("app", "Drive " << get_device_with_type() << " seems to be a RAID volume/controller.\n");
+		this->set_detected_type(detected_type_raid);
 	}
 
-	// Note: We don't use SmartctlParser here, because this information
-	// may be in some other format. If this information is valid, only then it's
-	// passed to SmartctlParser.
-	// Compared to SmartctlParser, this one is much more loose.
-
-	// Don't put complete messages here - they change across smartctl versions.
-	if (app_pcre_match("/^SMART support is:[ \\t]*Unavailable/mi", info_output_)  // cdroms output this
-			|| app_pcre_match("/Device does not support SMART/mi", info_output_)  // usb flash drives, non-smart hds
-			|| app_pcre_match("/Device Read Identity Failed/mi", info_output_)) {  // solaris scsi, unsupported by smartctl (maybe others?)
+	// RAID volume may report that it has SMART, but it obviously doesn't.
+	if (get_detected_type() == detected_type_raid) {
 		smart_supported_ = false;
 		smart_enabled_ = false;
 
-	} else if (app_pcre_match("/^SMART support is:[ \\t]*Available/mi", info_output_)
-			|| app_pcre_match("/^SMART support is:[ \\t]*Ambiguous/mi", info_output_)) {
-		smart_supported_ = true;
+	} else {
+		// Note: We don't use SmartctlParser here, because this information
+		// may be in some other format. If this information is valid, only then it's
+		// passed to SmartctlParser.
+		// Compared to SmartctlParser, this one is much looser.
 
-		if (app_pcre_match("/^SMART support is:[ \\t]*Enabled/mi", info_output_)) {
-			smart_enabled_ = true;
-		} else if (app_pcre_match("/^SMART support is:[ \\t]*Disabled/mi", info_output_)) {
+		// Don't put complete messages here - they change across smartctl versions.
+		if (app_pcre_match("/^SMART support is:[ \\t]*Unavailable/mi", info_output_)  // cdroms output this
+				|| app_pcre_match("/Device does not support SMART/mi", info_output_)  // usb flash drives, non-smart hds
+				|| app_pcre_match("/Device Read Identity Failed/mi", info_output_)) {  // solaris scsi, unsupported by smartctl (maybe others?)
+			smart_supported_ = false;
 			smart_enabled_ = false;
+
+		} else if (app_pcre_match("/^SMART support is:[ \\t]*Available/mi", info_output_)
+				|| app_pcre_match("/^SMART support is:[ \\t]*Ambiguous/mi", info_output_)) {
+			smart_supported_ = true;
+
+			if (app_pcre_match("/^SMART support is:[ \\t]*Enabled/mi", info_output_)) {
+				smart_enabled_ = true;
+			} else if (app_pcre_match("/^SMART support is:[ \\t]*Disabled/mi", info_output_)) {
+				smart_enabled_ = false;
+			}
 		}
 	}
 
-
 	std::string model;
 	if (app_pcre_match("/^Device Model:[ \\t]*(.*)$/mi", info_output_, &model)) {  // HD's and cdroms
 		model_name_ = hz::string_remove_adjacent_duplicates_copy(hz::string_trim_copy(model), ' ');
@@ -244,6 +269,13 @@ std::string StorageDevice::parse_basic_data(bool do_set_properties, bool emit_si
 		serial_number_ = hz::string_remove_adjacent_duplicates_copy(hz::string_trim_copy(serial), ' ');
 	}
 
+	std::string rpm_str;
+	if (app_pcre_match("/^Rotation Rate:[ \\t]*(.*)$/mi", info_output_, &rpm_str)) {
+		int rpm = 0;
+		hz::string_is_numeric(rpm_str, rpm, false);
+		hdd_ = rpm > 0;
+	}
+
 
 	// Note: this property is present since 5.33.
 	std::string size;
@@ -257,12 +289,19 @@ std::string StorageDevice::parse_basic_data(bool do_set_properties, bool emit_si
 	// Note that this may try to parse data the second time (it may already have
 	// been parsed by parse_data() which failed at it).
 	if (do_set_properties) {
+		StorageAttribute::DiskType disk_type = StorageAttribute::DiskAny;
+		if (hdd_.defined()) {
+			disk_type = hdd_.value() ? StorageAttribute::DiskHDD : StorageAttribute::DiskSSD;
+		}
 		SmartctlParser ps;
-		if (ps.parse_full(this->info_output_)) {  // try to parse it
+		if (ps.parse_full(this->info_output_, disk_type)) {  // try to parse it
 			this->set_properties(ps.get_properties());  // copy to our drive, overwriting old data
 		}
 	}
 
+	// A model field (and its aliases) is a good indication whether there was any data or not
+	set_parse_status(model_name_.defined() ? parse_status_info : parse_status_none);
+
 	if (emit_signal)
 		signal_changed.emit(this);  // notify listeners
 
@@ -314,8 +353,12 @@ std::string StorageDevice::parse_data()
 {
 	this->clear_fetched(false);  // clear everything fetched before, except outputs
 
+	StorageAttribute::DiskType disk_type = StorageAttribute::DiskAny;
+	if (hdd_.defined()) {
+		disk_type = hdd_.value() ? StorageAttribute::DiskHDD : StorageAttribute::DiskSSD;
+	}
 	SmartctlParser ps;
-	if (ps.parse_full(this->full_output_)) {  // try to parse it (parse only, set the properties after basic parsing).
+	if (ps.parse_full(this->full_output_, disk_type)) {  // try to parse it (parse only, set the properties after basic parsing).
 
 		// refresh basic info too
 		this->info_output_ = ps.get_data_full();  // put data including version information
@@ -325,13 +368,15 @@ std::string StorageDevice::parse_data()
 		// but this one sets the StorageDevice class members, not properties.
 		this->parse_basic_data(false, false);  // don't emit signal, we're not complete yet.
 
+		// Call this after parse_basic_data(), since it sets parse status to "info".
+		this->set_parse_status(StorageDevice::parse_status_full);
+
 		// set the full properties
-		this->set_fully_parsed(true);
 		this->set_properties(ps.get_properties());  // copy to our drive, overwriting old data
 
 		signal_changed.emit(this);  // notify listeners
 
-		return "";
+		return std::string();
 	}
 
 	// Don't show any GUI warnings on parse failure - it may just be an unsupported
@@ -339,22 +384,20 @@ std::string StorageDevice::parse_data()
 	// parsed again in Info window, and we show the warnings there.
 	debug_out_warn("app", DBG_FUNC_MSG << "Cannot parse smartctl output.\n");
 
-	this->set_fully_parsed(false);
-
 	// proper parsing failed. try to at least extract info section
 	this->info_output_ = this->full_output_;  // complete output here. sometimes it's only the info section
 	if (!this->parse_basic_data(true).empty()) {  // will add some properties too. this will emit signal_changed.
 		return ps.get_error_msg();  // return full parser's error messages - they are more detailed.
 	}
 
-	return "";  // return ok if at least the info was ok.
+	return std::string();  // return ok if at least the info was ok.
 }
 
 
 
-bool StorageDevice::get_fully_parsed() const
+StorageDevice::parse_status_t StorageDevice::get_parse_status() const
 {
-	return fully_parsed_;
+	return parse_status_;
 }
 
 
@@ -532,14 +575,14 @@ StorageProperty StorageDevice::get_health_property() const
 
 
 
-string StorageDevice::get_device() const
+std::string StorageDevice::get_device() const
 {
 	return device_;
 }
 
 
 
-string StorageDevice::get_device_base() const
+std::string StorageDevice::get_device_base() const
 {
 	if (is_virtual_)
 		return "";
@@ -552,23 +595,14 @@ string StorageDevice::get_device_base() const
 
 
 
-string StorageDevice::get_device_pretty() const
+std::string StorageDevice::get_device_with_type() const
 {
-	std::string ret;
 	if (this->get_is_virtual()) {
-		ret = "Virtual";
+		std::string ret = "Virtual";
 		std::string vf = this->get_virtual_filename();
 		ret += (" (" + (vf.empty() ? "[empty]" : vf) + ")");
-	} else {
-		ret = this->get_device_with_type();
+		return ret;
 	}
-	return ret;
-}
-
-
-
-string StorageDevice::get_device_with_type() const
-{
 	std::string device = get_device();
 	if (!get_type_argument().empty()) {
 		device += " (" + get_type_argument() + ")";
@@ -592,7 +626,7 @@ StorageDevice::detected_type_t StorageDevice::get_detected_type() const
 
 
 
-void StorageDevice::set_type_argument(const string& arg)
+void StorageDevice::set_type_argument(const std::string& arg)
 {
 	type_arg_ = arg;
 }
@@ -606,7 +640,7 @@ std::string StorageDevice::get_type_argument() const
 
 
 
-void StorageDevice::set_extra_arguments(const string& args)
+void StorageDevice::set_extra_arguments(const std::string& args)
 {
 	extra_args_ = args;
 }
@@ -627,7 +661,7 @@ bool StorageDevice::get_is_virtual() const
 
 
 
-string StorageDevice::get_virtual_file() const
+std::string StorageDevice::get_virtual_file() const
 {
 	return (is_virtual_ ? virtual_file_ : std::string());
 }
@@ -648,7 +682,7 @@ const SmartctlParser::prop_list_t& StorageDevice::get_properties() const
 
 
 
-StorageProperty StorageDevice::lookup_property(const string& generic_name, StorageProperty::section_t section, StorageProperty::subsection_t subsection) const
+StorageProperty StorageDevice::lookup_property(const std::string& generic_name, StorageProperty::section_t section, StorageProperty::subsection_t subsection) const
 {
 	for (SmartctlParser::prop_list_t::const_iterator iter = properties_.begin(); iter != properties_.end(); ++iter) {
 		if (section != StorageProperty::section_unknown && iter->section != section)
@@ -664,49 +698,56 @@ StorageProperty StorageDevice::lookup_property(const string& generic_name, Stora
 
 
 
-string StorageDevice::get_model_name() const
+std::string StorageDevice::get_model_name() const
 {
 	return (model_name_.defined() ? model_name_.value() : "");
 }
 
 
 
-string StorageDevice::get_family_name() const
+std::string StorageDevice::get_family_name() const
 {
 	return (family_name_.defined() ? family_name_.value() : "");
 }
 
 
 
-string StorageDevice::get_serial_number() const
+std::string StorageDevice::get_serial_number() const
 {
 	return (serial_number_.defined() ? serial_number_.value() : "");
 }
 
 
 
-void StorageDevice::set_info_output(const string& s)
+bool StorageDevice::get_is_hdd() const
+{
+	return hdd_.defined() ? hdd_.value() : false;
+}
+
+
+
+void StorageDevice::set_info_output(const std::string& s)
 {
 	info_output_ = s;
 }
 
 
 
-string StorageDevice::get_info_output() const
+std::string StorageDevice::get_info_output() const
 {
 	return info_output_;
 }
 
 
 
-void StorageDevice::set_full_output(const string& s)
+void StorageDevice::set_full_output(const std::string& s)
 {
 	full_output_ = s;
 }
 
 
 
-string StorageDevice::get_full_output() const
+std::string StorageDevice::get_full_output() const
 {
 	return full_output_;
 }
@@ -809,7 +850,7 @@ std::string StorageDevice::execute_device_smartctl(const std::string& command_op
 			command_options, smartctl_ex, smartctl_output);
 
 	if (!error_msg.empty()) {
-		debug_out_warn("app", DBG_FUNC_MSG << "Error while executing smartctl binary.\n");
+		debug_out_warn("app", DBG_FUNC_MSG << "Smartctl binary did not execute cleanly.\n");
 
 		// Smartctl 5.39 cvs/svn version defaults to usb type on at least linux and windows.
 		// This means that the old SCSI identify command isn't executed by default,
@@ -828,9 +869,9 @@ std::string StorageDevice::execute_device_smartctl(const std::string& command_op
 
 
 
-void StorageDevice::set_fully_parsed(bool b)
+void StorageDevice::set_parse_status(parse_status_t value)
 {
-	fully_parsed_ = b;
+	parse_status_ = value;
 }
 
 
@@ -845,3 +886,5 @@ void StorageDevice::set_properties(const SmartctlParser::prop_list_t& props)
 
 
 
+
+/// @}
