@@ -1,24 +1,27 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup gsc
+/// \weakgroup gsc
+/// @{
 
 #ifndef GSC_ABOUT_DIALOG_H
 #define GSC_ABOUT_DIALOG_H
 
-#include <gtkmm/window.h>
-#include <gtkmm/aboutdialog.h>
+#include <gtkmm.h>
 
 #include "applib/app_ui_res_utils.h"
 
 
 
 
-// use create() / destroy() with this class instead of new / delete!
-
+/// The About dialog.
+/// Use create() / destroy() with this class instead of new / delete!
 class GscAboutDialog : public AppUIResWidget<GscAboutDialog, false, Gtk::AboutDialog> {
-
 	public:
 
 		// name of glade/ui file without a .glade/.ui extension and quotes
@@ -32,10 +35,10 @@ class GscAboutDialog : public AppUIResWidget<GscAboutDialog, false, Gtk::AboutDi
 		HZ_RES_DATA_INIT_NAMED(AUTHORS_txt, "AUTHORS.txt", AuthorsTextResData);
 
 
-		// glade/gtkbuilder needs this constructor
+		/// Constructor, gtkbuilder/glade needs this.
 		GscAboutDialog(BaseObjectType* gtkcobj, const app_ui_res_ref_t& ref_ui);
 
-
+		/// Virtual destructor
 		virtual ~GscAboutDialog()
 		{ }
 
@@ -45,14 +48,13 @@ class GscAboutDialog : public AppUIResWidget<GscAboutDialog, false, Gtk::AboutDi
 
 		// -------------------- Callbacks
 
-		void on_activate_url(Gtk::AboutDialog& about_dialog, const Glib::ustring& link);
-
-		void on_activate_email(Gtk::AboutDialog& about_dialog, const Glib::ustring& link);
-
 
+		/// Callback - dialog response
 		void on_response_before(int response_id);
 
 
+		bool on_activate_link_before(const std::string& uri);
+
 		// ---------- override virtual methods
 
 		// we use .run(), so we don't need this
@@ -73,3 +75,5 @@ class GscAboutDialog : public AppUIResWidget<GscAboutDialog, false, Gtk::AboutDi
 
 
 #endif
+
+/// @}
