@@ -1,14 +1,18 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup gsc
+/// \weakgroup gsc
+/// @{
 
 #include <vector>
-#include <gtkmm/button.h>
-#include <gtkmm/textview.h>
-#include <gtkmm/treeview.h>
-#include <gdk/gdkkeysyms.h>  // GDK_Escape
+#include <gtkmm.h>
+#include <gdk/gdk.h>  // GDK_KEY_Escape
+#include "applib/app_gtkmm_features.h"
 
 #include "hz/string_algo.h"
 #include "applib/app_gtkmm_utils.h"  // app_gtkmm_create_tree_view_column
@@ -37,7 +41,7 @@ GscHelpWindow::GscHelpWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& re
 
 	Glib::RefPtr<Gtk::AccelGroup> accel_group = this->get_accel_group();
 	if (window_close_button) {
-		window_close_button->add_accelerator("clicked", accel_group, GDK_Escape,
+		window_close_button->add_accelerator("clicked", accel_group, GDK_KEY_Escape,
 				Gdk::ModifierType(0), Gtk::AccelFlags(0));
 	}
 
@@ -47,11 +51,9 @@ GscHelpWindow::GscHelpWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& re
 	Gtk::TreeView* treeview = this->lookup_widget<Gtk::TreeView*>("topics_treeview");
 	if (treeview) {
 		Gtk::TreeModelColumnRecord model_columns;
-		int num_tree_cols = 0;
-
 		// Topic
 		model_columns.add(col_topic);
-		num_tree_cols = app_gtkmm_create_tree_view_column(col_topic, *treeview, "Topic", "Topic");
+		app_gtkmm_create_tree_view_column(col_topic, *treeview, "Topic", "Topic");
 
 		// create a TreeModel (ListStore)
 		list_store = Gtk::ListStore::create(model_columns);
@@ -185,6 +187,20 @@ void GscHelpWindow::set_topic(const Glib::ustring& topic)
 
 
 
+bool GscHelpWindow::on_delete_event_before(GdkEventAny* e)
+{
+	destroy(this);
+	return true;  // event handled, don't call default virtual handler
+}
+
+
+
+void GscHelpWindow::on_window_close_button_clicked()
+{
+	destroy(this);
+}
+
+
 
 void GscHelpWindow::on_tree_selection_changed()
 {
@@ -203,3 +219,5 @@ void GscHelpWindow::on_tree_selection_changed()
 
 
 
+
+/// @}
