@@ -15,14 +15,9 @@
 #include <cstdio>  // std::printf
 #include <vector>
 #include <sstream>
-#include <gtkmm/main.h>
-#include <gtkmm/messagedialog.h>
-#include <gtk/gtk.h>  // gtk_window_set_default_icon_name, gtk_icon_theme_*
+#include <gtkmm.h>
 #include <glib.h>  // g_, G*
-#include <glibmm/refptr.h>
-#include <glibmm/miscutils.h>  // set_application_name
-#include <glibmm/thread.h>  // thread_init
-// #include <gtkmm.h>
+#include <glibmm.h>  // set_application_name
 // #include <iostream>
 
 #include "hz/hz_config.h"  // ENABLE_GLIB, VERSION, DEBUG_BUILD
@@ -35,6 +30,7 @@
 #include "hz/locale_tools.h"  // locale_c*
 #include "hz/string_algo.h"  // string_join()
 #include "hz/win32_tools.h"  // win32_get_registry_value_string()
+#include "hz/env_tools.h"
 
 #include "gsc_main_window.h"
 #include "gsc_executor_log_window.h"
@@ -295,7 +291,7 @@ namespace {
 		std::string warningtext = std::string("\nWarning: GSmartControl");
 		warningtext += " comes with ABSOLUTELY NO WARRANTY.\n";
 		warningtext += "See LICENSE_gsmartcontrol.txt file for details.\n";
-		warningtext += "\nCopyright (C) 2008 - 2012  Alexander Shaduri <ashaduri" "" "@" "" "" "gmail.com>\n\n";
+		warningtext += "\nCopyright (C) 2008 - 2017  Alexander Shaduri <ashaduri" "" "@" "" "" "gmail.com>\n\n";
 
 		std::fprintf(stdout, "%s%s", versiontext.c_str(), warningtext.c_str());
 	}
@@ -310,6 +306,10 @@ bool app_init_and_loop(int& argc, char**& argv)
 	// initialize GThread (for mutexes, etc... to work). Must be called before any other glib function.
 // 	Glib::thread_init();
 
+#ifdef _WIN32
+	// Disable client-side decorations (enable native windows decorations) under Windows.
+	hz::env_set_value("GTK_CSD", "0");
+#endif
 
 	// Glib needs the C locale set to system locale for command line args.
 	// We will reset it later if needed.
@@ -458,19 +458,18 @@ bool app_init_and_loop(int& argc, char**& argv)
 #ifndef _WIN32  // win32 version has its icon compiled-in, so no need to set it there.
 	{
 		// we load it via icontheme to provide multi-size version.
-		GtkIconTheme* default_icon_theme = gtk_icon_theme_get_default();
 
 		// application-installed, /usr/share/icons/<theme_name>/apps/<size>
-		if (gtk_icon_theme_has_icon(default_icon_theme, "gsmartcontrol")) {
-			gtk_window_set_default_icon_name("gsmartcontrol");
+		if (Gtk::IconTheme::get_default()->has_icon("gsmartcontrol")) {
+			Gtk::Window::set_default_icon_name("gsmartcontrol");
 
 		// try the gnome icon, it's higher quality / resolution
-		} else if (gtk_icon_theme_has_icon(default_icon_theme, "gnome-dev-harddisk")) {
-			gtk_window_set_default_icon_name("gnome-dev-harddisk");
+		} else if (Gtk::IconTheme::get_default()->has_icon("gnome-dev-harddisk")) {
+			Gtk::Window::set_default_icon_name("gnome-dev-harddisk");
 
 		// gtk built-in, always available
 		} else {
-			gtk_window_set_default_icon_name("gtk-harddisk");
+			Gtk::Window::set_default_icon_name("gtk-harddisk");
 		}
 	}
 #endif
