@@ -1,16 +1,21 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup gsc
+/// \weakgroup gsc
+/// @{
 
 #include <gtkmm.h>
-#include <algorithm>  // std::sort
 #include <vector>
 
 #include "hz/string_algo.h"  // string_split
 #include "hz/fs_file.h"  // hz::File
 #include "hz/debug.h"
+#include "hz/scoped_ptr.h"
 #include "rconfig/rconfig_mini.h"
 #include "applib/storage_detector.h"
 #include "applib/smartctl_parser.h"
@@ -120,11 +125,20 @@ GscMainWindow::GscMainWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& re
 
 
 
+void GscMainWindow::obj_destroy()
+{
+	// This is needed because for some reason, if any icon is selected,
+	// on_iconview_selection_changed() is called even after the window is deleted,
+	// causing crash on exit.
+	iconview->clear_all();
+}
+
+
 
 void GscMainWindow::populate_iconview(bool smartctl_valid)
 {
 	if (!smartctl_valid) {
-		iconview->empty_view_message = GscMainWindowIconView::message_no_smartctl;
+		iconview->set_empty_view_message(GscMainWindowIconView::message_no_smartctl);
 		iconview->clear_all();  // the message won't be shown without invalidating the region.
 		while (Gtk::Main::events_pending())  // give expose event the time it needs
 			Gtk::Main::iteration();
@@ -134,7 +148,7 @@ void GscMainWindow::populate_iconview(bool smartctl_valid)
 		rescan_devices();  // scan for devices and fill the iconview
 
 	} else {
-		iconview->empty_view_message = GscMainWindowIconView::message_scan_disabled;
+		iconview->set_empty_view_message(GscMainWindowIconView::message_scan_disabled);
 		iconview->clear_all();  // the message won't be shown without invalidating the region.
 		while (Gtk::Main::events_pending())  // give expose event the time it needs
 			Gtk::Main::iteration();
@@ -264,24 +278,20 @@ bool GscMainWindow::create_widgets()
 
 	actiongroup_main->add(Gtk::Action::create("device_menu", "_Device"));
 
-#if APP_GTKMM_CHECK_VERSION(2, 8, 0)  // Gtk::Stock::INFO is since gtkmm 2.8
 		action = Gtk::Action::create(APP_ACTION_NAME(action_view_details), Gtk::Stock::INFO, "_View details",
 				"View detailed information");
-#else
-		action = Gtk::Action::create(APP_ACTION_NAME(action_view_details), "_View details", "View detailed information");
-#endif
 		actiongroup_device->add((action_map[action_view_details] = action), Gtk::AccelKey("<control>V"),
 				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_view_details));
 
 		action = Gtk::ToggleAction::create(APP_ACTION_NAME(action_enable_smart), "Enable S_MART",
 				"Toggle SMART status. The status will be preserved at least until reboot (unless you toggle it again).");
-		action->connect_proxy(*lookup_widget<Gtk::CheckButton*>("status_smart_enabled_check"));
+		lookup_widget<Gtk::CheckButton*>("status_smart_enabled_check")->set_related_action(action);
 		actiongroup_device->add((action_map[action_enable_smart] = action), Gtk::AccelKey("<control>M"),
 				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_enable_smart));
 
 		action = Gtk::ToggleAction::create(APP_ACTION_NAME(action_enable_aodc), "Enable Auto O_ffline Data Collection",
 				"Toggle Automatic Offline Data Collection which will update \"offline\" SMART attributes every four hours");
-		action->connect_proxy(*lookup_widget<Gtk::CheckButton*>("status_aodc_enabled_check"));
+		lookup_widget<Gtk::CheckButton*>("status_aodc_enabled_check")->set_related_action(action);
 		actiongroup_device->add((action_map[action_enable_aodc] = action), Gtk::AccelKey("<control>F"),
 				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_enable_aodc));
 
@@ -377,14 +387,17 @@ bool GscMainWindow::create_widgets()
 	// add some more accelerators (in addition to existing ones)
 	Gtk::Widget* rescan_item = ui_manager->get_widget("/main_menubar/device_menu/" APP_ACTION_NAME(action_rescan_devices));
 	if (rescan_item)
-		rescan_item->add_accelerator("activate", get_accel_group(), GDK_F5, Gdk::ModifierType(0), Gtk::AccelFlags(0));
+		rescan_item->add_accelerator("activate", get_accel_group(), GDK_KEY_F5, Gdk::ModifierType(0), Gtk::AccelFlags(0));
 
 
 	// look after the created widgets
-	Gtk::VBox* menubar_vbox = lookup_widget<Gtk::VBox*>("menubar_vbox");
+	Gtk::Box* menubar_vbox = lookup_widget<Gtk::Box*>("menubar_vbox");
 	Gtk::Widget* menubar = ui_manager->get_widget("/main_menubar");
-	if (menubar_vbox && menubar)
-		menubar_vbox->pack_start(*menubar, Gtk::PACK_SHRINK);
+	if (menubar_vbox && menubar) {
+		menubar_vbox->pack_start(*menubar, Gtk::PACK_EXPAND_WIDGET);
+		menubar->set_hexpand(true);
+// 		menubar->set_halign(Gtk::ALIGN_START);
+	}
 
 
 	// Set tooltips on menu items - gtk does that only on toolbar items.
@@ -406,25 +419,28 @@ bool GscMainWindow::create_widgets()
 	// ----------------------------------------- Labels
 
 	// create and add labels
-	Gtk::HBox* name_label_box = lookup_widget<Gtk::HBox*>("status_name_label_hbox");
+	Gtk::Box* name_label_box = lookup_widget<Gtk::Box*>("status_name_label_hbox");
 	if (name_label_box) {
-		name_label = Gtk::manage(new WrappingLabel("No drive selected", Gtk::ALIGN_LEFT));
+		name_label = Gtk::manage(new Gtk::Label("No drive selected", Gtk::ALIGN_START));
+		name_label->set_line_wrap(true);
 		name_label->set_selectable(true);
 		name_label->show();
 		name_label_box->pack_start(*name_label, true, true);
 	}
 
-	Gtk::HBox* health_label_box = lookup_widget<Gtk::HBox*>("status_health_label_hbox");
+	Gtk::Box* health_label_box = lookup_widget<Gtk::Box*>("status_health_label_hbox");
 	if (health_label_box) {
-		health_label = Gtk::manage(new WrappingLabel("No drive selected", Gtk::ALIGN_LEFT));
+		health_label = Gtk::manage(new Gtk::Label("No drive selected", Gtk::ALIGN_START));
+		health_label->set_line_wrap(true);
 		health_label->set_selectable(true);
 		health_label->show();
 		health_label_box->pack_start(*health_label, true, true);
 	}
 
-	Gtk::HBox* family_label_box = lookup_widget<Gtk::HBox*>("status_family_label_hbox");
+	Gtk::Box* family_label_box = lookup_widget<Gtk::Box*>("status_family_label_hbox");
 	if (family_label_box) {
-		family_label = Gtk::manage(new WrappingLabel("No drive selected", Gtk::ALIGN_LEFT));
+		family_label = Gtk::manage(new Gtk::Label("No drive selected", Gtk::ALIGN_START));
+		family_label->set_line_wrap(true);
 		family_label->set_selectable(true);
 		family_label->show();
 		family_label_box->pack_start(*family_label, true, true);
@@ -563,7 +579,9 @@ void GscMainWindow::on_action_activated(GscMainWindow::action_t action_type)
 		case action_preferences:
 		{
 			GscPreferencesWindow* win = GscPreferencesWindow::create();  // destroyed on close
-			// win->set_transient_for(*this);  // for "destroy with parent", always-on-top
+			win->set_transient_for(*this);  // for "destroy with parent", always-on-top
+			win->set_main_window(this);
+			win->set_modal(true);
 			win->show();
 			break;
 		}
@@ -572,8 +590,11 @@ void GscMainWindow::on_action_activated(GscMainWindow::action_t action_type)
 		{
 			GscHelpWindow* win = GscHelpWindow::create();  // destroyed on close
 			// win->set_transient_for(*this);  // for "destroy with parent", always-on-top
-			win->set_topic("Begin");  // special mark, top of the text.
 			win->show();
+			while(g_main_context_pending(NULL)) {
+				g_main_context_iteration(NULL, false);
+			}
+			win->set_topic("Begin");  // special mark, top of the text.
 			break;
 		}
 
@@ -581,8 +602,11 @@ void GscMainWindow::on_action_activated(GscMainWindow::action_t action_type)
 		{
 			GscHelpWindow* win = GscHelpWindow::create();  // destroyed on close
 			// win->set_transient_for(*this);  // for "destroy with parent", always-on-top
-			win->set_topic("Permission Problems");
 			win->show();
+			while(g_main_context_pending(NULL)) {
+				g_main_context_iteration(NULL, false);
+			}
+			win->set_topic("Permission Problems");
 			break;
 		}
 
@@ -590,8 +614,11 @@ void GscMainWindow::on_action_activated(GscMainWindow::action_t action_type)
 		{
 			GscHelpWindow* win = GscHelpWindow::create();  // destroyed on close
 			// win->set_transient_for(*this);  // for "destroy with parent", always-on-top
-			win->set_topic("Enable SMART Permanently");
 			win->show();
+			while(g_main_context_pending(NULL)) {
+				g_main_context_iteration(NULL, false);
+			}
+			win->set_topic("Enable SMART Permanently");
 			break;
 		}
 
@@ -599,8 +626,11 @@ void GscMainWindow::on_action_activated(GscMainWindow::action_t action_type)
 		{
 			GscHelpWindow* win = GscHelpWindow::create();  // destroyed on close
 			// win->set_transient_for(*this);  // for "destroy with parent", always-on-top
-			win->set_topic("Reporting Bugs");
 			win->show();
+			while(g_main_context_pending(NULL)) {
+				g_main_context_iteration(NULL, false);
+			}
+			win->set_topic("Reporting Bugs");
 			break;
 		}
 
@@ -698,7 +728,7 @@ void GscMainWindow::on_action_enable_aodc_toggled(Gtk::ToggleAction* action)
 
 			Gtk::Button enable_button("_Enable", true);
 			enable_button.set_image(*Gtk::manage(new Gtk::Image(Gtk::Stock::YES, Gtk::ICON_SIZE_BUTTON)));
-			enable_button.set_flags(enable_button.get_flags() | Gtk::CAN_DEFAULT);
+			enable_button.set_can_default(true);
 			enable_button.show_all();
 			dialog.add_action_widget(enable_button, Gtk::RESPONSE_YES);
 			enable_button.grab_default();  // make it the default widget
@@ -1003,7 +1033,7 @@ void GscMainWindow::rescan_devices()
 // 	hz::string_split(match_str, ';', match_patterns, true);
 	hz::string_split(blacklist_str, ';', blacklist_patterns, true);
 
-	iconview->empty_view_message = GscMainWindowIconView::message_scanning;
+	iconview->set_empty_view_message(GscMainWindowIconView::message_scanning);
 
 	iconview->clear_all();  // clear previous icons, invalidate region to update the message.
 	while (Gtk::Main::events_pending())  // give expose event the time it needs
@@ -1045,8 +1075,6 @@ void GscMainWindow::rescan_devices()
 
 	// add them anyway, in case the error was only on one drive.
 	} else { // if (!error) {
-		std::sort(drives.begin(), drives.end());  // hard drives first
-
 		// add them to iconview
 		for (unsigned int i = 0; i < drives.size(); ++i) {
 			if (rconfig::get_data<bool>("gui/show_smart_capable_only")) {
@@ -1059,8 +1087,8 @@ void GscMainWindow::rescan_devices()
 	}
 
 	// in case there are no drives in the system.
-	if (iconview->num_icons == 0)
-		iconview->empty_view_message = GscMainWindowIconView::message_no_drives_found;
+	if (iconview->get_num_icons() == 0)
+		iconview->set_empty_view_message(GscMainWindowIconView::message_no_drives_found);
 
 	this->scanning_ = false;
 }
@@ -1187,8 +1215,8 @@ GscInfoWindow* GscMainWindow::show_device_info_window(StorageDeviceRefPtr drive)
 
 
 	// Virtual drives are parsed at load time.
-	// Parse non-virtual drives here.
-	if (!drive->get_is_virtual()) {
+	// Parse non-virtual, smart-supporting drives here.
+	if (!drive->get_is_virtual() && drive->get_smart_status() != StorageDevice::status_unsupported) {
 		SmartctlExecutorGuiRefPtr ex(new SmartctlExecutorGui());
 		ex->create_running_dialog(this, "Running %s on " + drive->get_device_with_type() + "...");
 		std::string error_msg = drive->fetch_data_and_parse(ex);  // run it with GUI support
@@ -1203,7 +1231,7 @@ GscInfoWindow* GscMainWindow::show_device_info_window(StorageDeviceRefPtr drive)
 	// If the drive output wasn't fully parsed (happens with e.g. scsi and
 	// usb devices), only very basic info is available and there's no point
 	// in showing this window. - for both virtual and non-virtual.
-	if (!drive->get_fully_parsed()) {
+	if (drive->get_parse_status() == StorageDevice::parse_status_none) {
 		gsc_no_info_dialog_show("No additional information is available for this drive.",
 				"", this, false, drive->get_info_output(), "Smartctl Output", drive->get_save_filename());
 		return 0;
@@ -1224,6 +1252,16 @@ GscInfoWindow* GscMainWindow::show_device_info_window(StorageDeviceRefPtr drive)
 
 
 
+void GscMainWindow::show_prefs_updated_message()
+{
+	iconview->set_empty_view_message(GscMainWindowIconView::message_please_rescan);
+	iconview->clear_all();  // the message won't be shown without invalidating the region.
+	while (Gtk::Main::events_pending())  // give expose event the time it needs
+		Gtk::Main::iteration();
+}
+
+
+
 
 void GscMainWindow::show_add_device_chooser()
 {
@@ -1238,7 +1276,19 @@ void GscMainWindow::show_add_device_chooser()
 void GscMainWindow::show_load_virtual_file_chooser()
 {
 	static std::string last_dir;
+	int result = 0;
 
+#if GTK_CHECK_VERSION(3, 20, 0)
+	hz::scoped_ptr<GtkFileChooserNative> dialog(gtk_file_chooser_native_new(
+			"Load Data From...", this->gobj(), GTK_FILE_CHOOSER_ACTION_OPEN, NULL, NULL), g_object_unref);
+
+	if (!last_dir.empty()) {
+		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog.get()), last_dir.c_str());
+	}
+
+	result = gtk_native_dialog_run(GTK_NATIVE_DIALOG(dialog.get()));
+
+#else
 	Gtk::FileChooserDialog dialog(*this, "Load Data From...",
 			Gtk::FILE_CHOOSER_ACTION_OPEN);
 
@@ -1250,15 +1300,22 @@ void GscMainWindow::show_load_virtual_file_chooser()
 		dialog.set_current_folder(last_dir);
 
 	// Show the dialog and wait for a user response
-	int result = dialog.run();  // the main cycle blocks here
+	result = dialog.run();  // the main cycle blocks here
+#endif
 
 	// Handle the response
 	switch (result) {
 		case Gtk::RESPONSE_ACCEPT:
 		{
-			last_dir = dialog.get_current_folder();  // safe for the future
+			std::string file;
 
-			std::string file = dialog.get_filename();  // in fs encoding
+#if GTK_CHECK_VERSION(3, 20, 0)
+			file = app_ustring_from_gchar(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog.get())));
+			last_dir = hz::path_get_dirname(file);
+#else
+			file = dialog.get_filename();  // in fs encoding
+			last_dir = dialog.get_current_folder();  // save for the future
+#endif
 			this->add_virtual_drive(file);
 			break;
 		}
@@ -1275,7 +1332,6 @@ void GscMainWindow::show_load_virtual_file_chooser()
 
 
 
-
 // by default, delete_event calls hide().
 bool GscMainWindow::on_delete_event_before(GdkEventAny* e)
 {
@@ -1293,3 +1349,5 @@ bool GscMainWindow::on_delete_event_before(GdkEventAny* e)
 
 
 
+
+/// @}
