@@ -1,53 +1,56 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup gsc
+/// \weakgroup gsc
+/// @{
 
 #ifndef GSC_MAIN_WINDOW_ICONVIEW_H
 #define GSC_MAIN_WINDOW_ICONVIEW_H
 
 #include <gtkmm.h>
-#include <gtkmm/icontheme.h>  // must include manually for older gtkmm (at least 2.6, but not 2.12)
 #include <vector>
 #include <cmath>  // std::floor
-
-#if APP_GTKMM_CHECK_VERSION(2, 8, 0)
-	#include <gtk/gtk.h>  // gtk_icon_view_set_cursor, GTK_ICON_VIEW
-#endif
+#include <cairomm/cairomm.h>
 
 #include "hz/string_algo.h"  // string_join
 #include "hz/debug.h"
 #include "hz/data_file.h"  // data_file_find
+#include "applib/app_gtkmm_utils.h"
+#include "applib/storage_property_colors.h"
 
 #include "gsc_main_window.h"
+#include "rconfig/rconfig_mini.h"
 
 
 
-
-// Note: The IconView must have a fixed icon width set (e.g. in glade file).
-// Otherwise, it doesn't re-compute it when clearing and adding new icons.
-
+/// The icon view of the main window (shows a drive list).
+/// Note: The IconView must have a fixed icon width set (e.g. in glade file).
+/// Otherwise, it doesn't re-compute it when clearing and adding new icons.
 class GscMainWindowIconView : public Gtk::IconView {
 	public:
 
-		typedef GscMainWindowIconView self_type;  // needed for CONNECT_VIRTUAL
+		typedef GscMainWindowIconView self_type;  ///< Self type, needed for CONNECT_VIRTUAL
 
+		/// Message type to show
 		enum message_t {
-			message_none,
-			message_scan_disabled,
-			message_scanning,
-			message_no_drives_found,
-			message_no_smartctl
+			message_none,  ///< No message
+			message_scan_disabled,  ///< Scanning is disabled
+			message_scanning,  ///< Scanning drives...
+			message_no_drives_found,  ///< No drives found
+			message_no_smartctl,  ///< No smartctl installed
+			message_please_rescan,  ///< Re-scan to see the drives
 		};
 
 
-		// glade/gtkbuilder needs this constructor
+		/// Constructor, gtkbuilder/glade needs this.
 		GscMainWindowIconView(BaseObjectType* gtkcobj, const app_ui_res_ref_t& ref_ui)
 				: Gtk::IconView(gtkcobj), num_icons(0), main_window(0), empty_view_message(message_none)
 		{
-			APP_GTKMM_CONNECT_VIRTUAL(expose_event);  // make sure the event handler is called
-
 			columns.add(col_name);  // we can use the col_name variable by value after this.
 			this->set_markup_column(col_name);
 
@@ -64,11 +67,16 @@ class GscMainWindowIconView : public Gtk::IconView {
 			this->set_model(ref_list_model);
 
 			// we add it here because list model must be created already
-			gtkmm_set_iconview_tooltip_column(this, col_description, ref_list_model);
+			set_tooltip_column(col_description.index());
 
 
 			// icons
-			Glib::RefPtr<Gtk::IconTheme> default_icon_theme = Gtk::IconTheme::get_default();
+			Glib::RefPtr<Gtk::IconTheme> default_icon_theme;
+			try {
+				default_icon_theme = Gtk::IconTheme::get_default();
+			} catch (...) {
+				// nothing
+			}
 
 			// Try Gnome icons icons first, they are usually more consistent with Gnome desktop
 			// (this should work only if gnome-settings-daemon is running, so no harm on other desktops).
@@ -91,7 +99,7 @@ class GscMainWindowIconView : public Gtk::IconView {
 			}
 
 			// last resort. gtk has gtk-harddisk built-in. it may not be the right size, but what else can we do?
-			if (!hd_icon)
+			if (!hd_icon && default_icon_theme)
 				hd_icon = default_icon_theme->load_icon("gtk-harddisk", 48, Gtk::IconLookupFlags(0));
 
 
@@ -110,7 +118,7 @@ class GscMainWindowIconView : public Gtk::IconView {
 				}
 			}
 
-			if (!cddvd_icon)
+			if (!cddvd_icon && default_icon_theme)
 				cddvd_icon = default_icon_theme->load_icon("gtk-cdrom", 48, Gtk::IconLookupFlags(0));
 
 
@@ -125,34 +133,42 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
-
+		/// Set the parent window
 		void set_main_window(GscMainWindow* w)
 		{
 			main_window = w;
 		}
 
 
-
-		// override default event handler
-		bool on_expose_event_before(GdkEventExpose* exp_event)
+		/// Set the message type to display when there are no icons to show
+		void set_empty_view_message(message_t message)
 		{
-			// this->on_expose_event(exp_event);  // parent's stuff
+			empty_view_message = message;
+		}
 
+
+		/// Get the number of icons currently displayed
+		unsigned int get_num_icons() const
+		{
+			return num_icons;
+		}
+
+
+		// Overridden from Gtk::Widget
+		bool on_draw(const Cairo::RefPtr<Cairo::Context>& cr)
+		{
 			if (empty_view_message != message_none && this->num_icons == 0) {  // no icons
-				Glib::RefPtr<Gdk::Drawable> win = Glib::wrap(exp_event->window, true);
-	// 			Cairo::RefPtr<Cairo::Context> cr = win->create_cairo_context();
-				Glib::RefPtr<Gdk::GC> gc = Gdk::GC::create(win);
-
 				std::string msg;
 				switch(empty_view_message) {
 					case message_scan_disabled: msg = "Automatic scanning is disabled.\nPress Ctrl+R to scan manually."; break;
 					case message_scanning: msg = "Scanning system, please wait..."; break;
 					case message_no_drives_found: msg = "No drives found."; break;
 					case message_no_smartctl: msg = "Please specify the correct smartctl binary in\nPreferences and press Ctrl-R to re-scan."; break;
+					case message_please_rescan: msg = "Preferences changed.\nPress Ctrl-R to re-scan."; break;
 					default: msg = "[error - invalid message]";
 				}
 
-				Glib::RefPtr<Pango::Layout> layout = create_pango_layout("");
+				Glib::RefPtr<Pango::Layout> layout = this->create_pango_layout("");
 				layout->set_alignment(Pango::ALIGN_CENTER);
 				layout->set_markup(msg);
 
@@ -161,15 +177,19 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 				int pos_x = (get_allocation().get_width() - layout_w) / 2;
 				int pos_y = (get_allocation().get_height() - layout_h) / 2;
+				cr->move_to(pos_x, pos_y);
 
-				win->draw_layout(gc, pos_x, pos_y, layout);
+				layout->show_in_cairo_context(cr);
+
+				return true;
 			}
 
-			return false;  // without false the icons don't get drawn...
+			return Gtk::IconView::on_draw(cr);
 		}
 
 
 
+		/// Add a drive entry to the icon view
 		void add_entry(StorageDeviceRefPtr drive, bool scroll_to_it = false)
 		{
 			if (!drive)
@@ -185,15 +205,13 @@ class GscMainWindowIconView : public Gtk::IconView {
 			if (scroll_to_it) {
 				Gtk::TreeModel::Path tpath(row);
 				// scroll_to_path() and set/get_cursor() are since gtkmm 2.8.
-#if APP_GTKMM_CHECK_VERSION(2, 8, 0)
+
 				this->scroll_to_path(tpath, true, 0.5, 0.5);
 				// select it (keyboard & selection)
 				Gtk::CellRenderer* cell = 0;
 				if (this->get_cursor(cell) && cell) {
-					// gtkmm's set_cursor() is undefined (but declared) in 2.8, so use gtk variant.
-					gtk_icon_view_set_cursor(GTK_ICON_VIEW(this->gobj()), tpath.gobj(), cell->gobj(), false);
+					this->set_cursor(tpath, *cell, false);
 				}
-#endif
 				this->select_path(tpath);  // highlight it
 			}
 
@@ -202,7 +220,8 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 
-		// this should be called to update the icon of already refreshed drive
+		/// Decorate a drive entry (colorize it if it has errors, etc...).
+		/// This should be called to update the icon of already refreshed drive.
 		void decorate_entry(Gtk::TreePath model_path)
 		{
 			if (model_path.empty())
@@ -214,17 +233,27 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 
-		// this should be called to update the icon of already refreshed drive
+		/// Decorate a drive entry (colorize it if it has errors, etc...).
+		/// This should be called to update the icon of already refreshed drive.
 		void decorate_entry(Gtk::TreeModel::Row& row)
 		{
 			StorageDeviceRefPtr drive = row[col_drive_ptr];
 
 			// it needs this space to be symmetric (why?);
-			std::string name;  // = "<big>" + drive->get_device_pretty() + " </big>\n";
+			std::string name;  // = "<big>" + drive->get_device_with_type() + " </big>\n";
 
 			// note: if this wraps, it becomes left-aligned in gtk <= 2.10.
-			name += (drive->get_model_name().empty() ? std::string("Unknown model") : drive->get_model_name());
-
+			name += (drive->get_model_name().empty() ? Glib::ustring("Unknown model") : Glib::Markup::escape_text(drive->get_model_name()));
+			if (rconfig::get_data<bool>("gui/icons_show_device_name")) {
+				if (!drive->get_is_virtual()) {
+					name += "\n" + Glib::Markup::escape_text(drive->get_device_with_type());
+				} else if (!drive->get_virtual_filename().empty()) {
+					name += "\n" + Glib::Markup::escape_text(drive->get_virtual_filename());
+				}
+			}
+			if (rconfig::get_data<bool>("gui/icons_show_serial_number") && !drive->get_serial_number().empty()) {
+				name += "\n" + Glib::Markup::escape_text(drive->get_serial_number());
+			}
 
 			std::vector<std::string> tooltip_strs;
 
@@ -234,6 +263,9 @@ class GscMainWindowIconView : public Gtk::IconView {
 			} else {
 				tooltip_strs.push_back("Device: <b>" + Glib::Markup::escape_text(drive->get_device_with_type()) + "</b>");
 			}
+			if (!drive->get_serial_number().empty()) {
+				tooltip_strs.push_back("Serial number: <b>" + Glib::Markup::escape_text(drive->get_serial_number()) + "</b>");
+			}
 			tooltip_strs.push_back("SMART status: <b>"
 					+ StorageDevice::get_status_name(drive->get_smart_status(), false) + "</b>");
 			tooltip_strs.push_back("Automatic Offline Data Collection status: <b>"
@@ -243,10 +275,15 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 			Glib::RefPtr<Gdk::Pixbuf> icon;
-			if (drive->get_detected_type() == StorageDevice::detected_type_cddvd) {
-				icon = cddvd_icon;
-			} else {
-				icon = hd_icon;
+			switch(drive->get_detected_type()) {
+				case StorageDevice::detected_type_cddvd:
+					icon = cddvd_icon;
+					break;
+				case StorageDevice::detected_type_unknown:  // standard HD icon
+				case StorageDevice::detected_type_invalid:
+				case StorageDevice::detected_type_raid:  // TODO a separate icon for this
+					icon = hd_icon;
+					break;
 			}
 
 			StorageProperty health_prop = drive->get_health_property();
@@ -291,6 +328,7 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 
+		/// Remove drive entry
 		void remove_entry(const Gtk::TreePath& model_path)
 		{
 			Gtk::TreeModel::Row row = *(ref_list_model->get_iter(model_path));
@@ -298,6 +336,8 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
+
+		/// Remove selected drive entry
 		void remove_selected_drive()
 		{
 			if (this->get_selected_items().size()) {
@@ -308,13 +348,14 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 
+		/// Remove all entries
 		void clear_all()
 		{
 			num_icons = 0;
 			ref_list_model->clear();
 
 			// this is needed to update the label from "disabled" to "scanning"
-			if (this->is_realized()) {
+			if (this->get_realized()) {
 				Gdk::Rectangle rect = this->get_allocation();
 				Glib::RefPtr<Gdk::Window> win = this->get_window();
 				win->invalidate_rect(rect, true);  // force expose event
@@ -323,6 +364,8 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
+
+		/// Get selected drive
 		StorageDeviceRefPtr get_selected_drive()
 		{
 			StorageDeviceRefPtr drive = 0;
@@ -335,6 +378,8 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
+
+		/// Get tree path by a drive
 		Gtk::TreePath get_path_by_drive(StorageDevice* drive)
 		{
 			Gtk::TreeNodeChildren children = ref_list_model->children();
@@ -349,6 +394,7 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 
+		/// Update menu actions in the Drives menu
 		void update_menu_actions()
 		{
 			// if there's nothing selected, disable items from "Drives" menu
@@ -365,7 +411,7 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
-		// callback
+		/// Callback
 		void on_iconview_item_activated(const Gtk::TreePath& model_path)
 		{
 			debug_out_info("app", DBG_FUNC << "\n");
@@ -379,7 +425,7 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
-		// callback
+		/// Callback
 		void on_iconview_selection_changed()
 		{
 			// Must do it here - if done during menu activation, the actions won't work
@@ -390,7 +436,7 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
-		// callback
+		/// Callback
 		bool on_iconview_button_press_event(GdkEventButton* event_button)
 		{
 			// select and show popup menu on right-click
@@ -405,14 +451,11 @@ class GscMainWindowIconView : public Gtk::IconView {
 				if (tpath.gobj() && !tpath.empty()) {  // without gobj() check gtkmm 2.6 (but not 2.12) prints lots of errors
 					// move keyboard focus to the icon (just as left-click does)
 
-					// set/get_cursor() are since gtkmm 2.8.
-#if APP_GTKMM_CHECK_VERSION(2, 8, 0)
 					Gtk::CellRenderer* cell = 0;
 					if (this->get_cursor(cell) && cell) {
 						// gtkmm's set_cursor() is undefined (but declared) in 2.8, so use gtk variant.
 						gtk_icon_view_set_cursor(GTK_ICON_VIEW(this->gobj()), tpath.gobj(), cell->gobj(), false);
 					}
-#endif
 
 					// select the icon
 					this->select_path(tpath);
@@ -437,7 +480,7 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
-		// callback attached to StorageDevice
+		/// Callback attached to StorageDevice, updates its view.
 		void on_drive_changed(StorageDevice* drive)
 		{
 			Gtk::TreePath model_path = this->get_path_by_drive(drive);
@@ -447,24 +490,25 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
+	private:
 
-		Gtk::TreeModel::ColumnRecord columns;
+		Gtk::TreeModel::ColumnRecord columns;  ///< Model columns
 
-		Gtk::TreeModelColumn<std::string> col_name;
-		Gtk::TreeModelColumn<Glib::ustring> col_description;
-		Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > col_pixbuf;
-		Gtk::TreeModelColumn<StorageDeviceRefPtr> col_drive_ptr;
+		Gtk::TreeModelColumn<std::string> col_name;  ///< Model column
+		Gtk::TreeModelColumn<Glib::ustring> col_description;  ///< Model column
+		Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > col_pixbuf;  ///< Model column
+		Gtk::TreeModelColumn<StorageDeviceRefPtr> col_drive_ptr;  ///< Model column
 
-		Glib::RefPtr<Gtk::ListStore> ref_list_model;
-		unsigned int num_icons;  // track the number of icons, because liststore makes it difficult to count them.
+		Glib::RefPtr<Gtk::ListStore> ref_list_model;  ///< The icon view model
+		unsigned int num_icons;  ///< Track the number of icons, because liststore makes it difficult to count them.
 
 		// available icons
-		Glib::RefPtr<Gdk::Pixbuf> hd_icon;
-		Glib::RefPtr<Gdk::Pixbuf> cddvd_icon;
+		Glib::RefPtr<Gdk::Pixbuf> hd_icon;  ///< Icon pixbuf
+		Glib::RefPtr<Gdk::Pixbuf> cddvd_icon;  ///< Icon pixbuf
 
-		GscMainWindow* main_window;
+		GscMainWindow* main_window;  ///< The main window, our parent
 
-		message_t empty_view_message;  // message displayed when no icons
+		message_t empty_view_message;  ///< Message type to display when not showing any icons
 
 };
 
@@ -476,3 +520,5 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 #endif
+
+/// @}
