@@ -1,47 +1,50 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup gsc
+/// \weakgroup gsc
+/// @{
 
 #ifndef GSC_TEXT_WINDOW_H
 #define GSC_TEXT_WINDOW_H
 
-#include <gtkmm/window.h>
-#include <gtkmm/button.h>
-#include <gtkmm/accelgroup.h>
-#include <gtkmm/textview.h>
-#include <gtkmm/filechooserdialog.h>
-#include <gdk/gdkkeysyms.h>  // GDK_Escape
+#include <gtkmm.h>
+#include <gdk/gdk.h>  // GDK_KEY_Escape
 
 #include "hz/debug.h"
 #include "hz/fs_file.h"
+#include "hz/scoped_ptr.h"
 
+#include "applib/app_gtkmm_features.h"
 #include "applib/app_ui_res_utils.h"
+#include "applib/app_gtkmm_utils.h"
 
 
 
+/// GscTextWindow InstanceSwitch parameter for smartctl output instance.
 struct SmartctlOutputInstance { static const bool multi_instance = true; };
 
 
 
-// use create() / destroy() with this class instead of new / delete!
-
-
-// InstanceSwitch: e.g. supply 3 different classes to use 3 different single-instance windows.
+/// A generic text-displaying window.
+/// Use create() / destroy() with this class instead of new / delete!
+/// \tparam InstanceSwitch e.g. supply 3 different classes to use 3 different single-instance windows.
 template<class InstanceSwitch>
 class GscTextWindow : public AppUIResWidget<GscTextWindow<InstanceSwitch>, InstanceSwitch::multi_instance> {
-
 	public:
 
 		// name of glade/ui file without a .glade/.ui extension and quotes
 		APP_UI_RES_DATA_INIT(gsc_text_window);
 
-		// needed for glade, not inherited from parent because of templates
+		/// Self type, needed for glade, not inherited from parent because of templates
 		typedef GscTextWindow<InstanceSwitch> self_type;
 
 
-		// glade/gtkbuilder needs this constructor
+		/// Constructor, gtkbuilder/glade needs this.
 		GscTextWindow(typename Gtk::Window::BaseObjectType* gtkcobj, const app_ui_res_ref_t& ref_ui)
 				: AppUIResWidget<GscTextWindow<InstanceSwitch>, InstanceSwitch::multi_instance>(gtkcobj, ref_ui)
 		{
@@ -60,23 +63,22 @@ class GscTextWindow : public AppUIResWidget<GscTextWindow<InstanceSwitch>, Insta
 
 			Glib::RefPtr<Gtk::AccelGroup> accel_group = this->get_accel_group();
 			if (close_window_button) {
-				close_window_button->add_accelerator("clicked", accel_group, GDK_Escape,
+				close_window_button->add_accelerator("clicked", accel_group, GDK_KEY_Escape,
 						Gdk::ModifierType(0), Gtk::AccelFlags(0));
 			}
 
-
-			// ---------------
-
 			// show();  // better show later, after set_text().
 
 			default_title_ = this->get_title();
 		}
 
 
+		/// Virtual destructor
 		virtual ~GscTextWindow()
 		{ }
 
 
+		/// Set the text to display
 		void set_text(const Glib::ustring& title, const Glib::ustring& contents,
 				bool save_visible = false, bool use_monospace = false)
 		{
@@ -87,7 +89,7 @@ class GscTextWindow : public AppUIResWidget<GscTextWindow<InstanceSwitch>, Insta
 			Gtk::TextView* textview = this->template lookup_widget<Gtk::TextView*>("main_textview");
 			if (textview) {
 				Glib::RefPtr<Gtk::TextBuffer> buffer = textview->get_buffer();
-				buffer->set_text(contents);
+				buffer->set_text(app_output_make_valid(contents));
 
 				if (use_monospace) {
 					Glib::RefPtr<Gtk::TextTag> tag = buffer->create_tag();
@@ -109,6 +111,7 @@ class GscTextWindow : public AppUIResWidget<GscTextWindow<InstanceSwitch>, Insta
 		}
 
 
+		/// Set the default file name to be shown on Save As
 		void set_save_filename(const std::string& filename)
 		{
 			save_filename_ = filename;
@@ -118,25 +121,43 @@ class GscTextWindow : public AppUIResWidget<GscTextWindow<InstanceSwitch>, Insta
 	protected:
 
 
-		// -------------------- Callbacks
+		// -------------------- callbacks
 
 
-		// ---------- override virtual methods
+		// ---------- overriden virtual methods
 
-		// by default, delete_event calls hide().
+		/// Destroy this object on delete event (by default it calls hide()).
+		/// Reimplemented from Gtk::Window.
 		bool on_delete_event_before(GdkEventAny* e)
 		{
-			destroy(this);  // deletes this object and nullifies instance
+			this->destroy(this);  // deletes this object and nullifies instance
 			return true;  // event handled, don't call default virtual handler
 		}
 
 
 		// ---------- other callbacks
 
+		/// Button click callback
 		void on_save_as_button_clicked()
 		{
 			static std::string last_dir;
+			int result = 0;
 
+#if GTK_CHECK_VERSION(3, 20, 0)
+			hz::scoped_ptr<GtkFileChooserNative> dialog(gtk_file_chooser_native_new(
+					"Save Data As...", this->gobj(), GTK_FILE_CHOOSER_ACTION_SAVE, NULL, NULL), g_object_unref);
+
+			gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog.get()), true);
+
+			if (!last_dir.empty())
+				gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog.get()), last_dir.c_str());
+
+			if (!save_filename_.empty())
+				gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog.get()), save_filename_.c_str());
+
+			result = gtk_native_dialog_run(GTK_NATIVE_DIALOG(dialog.get()));
+
+#else
 			Gtk::FileChooserDialog dialog(*this, "Save Data As...",
 					Gtk::FILE_CHOOSER_ACTION_SAVE);
 
@@ -144,9 +165,7 @@ class GscTextWindow : public AppUIResWidget<GscTextWindow<InstanceSwitch>, Insta
 			dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
 			dialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_ACCEPT);
 
-#if APP_GTKMM_CHECK_VERSION(2, 8, 0)
-			dialog.set_do_overwrite_confirmation(true);  // since gtkmm 2.8
-#endif
+			dialog.set_do_overwrite_confirmation(true);
 
 			if (!last_dir.empty())
 				dialog.set_current_folder(last_dir);
@@ -155,15 +174,21 @@ class GscTextWindow : public AppUIResWidget<GscTextWindow<InstanceSwitch>, Insta
 				dialog.set_current_name(save_filename_);
 
 			// Show the dialog and wait for a user response
-			int result = dialog.run();  // the main cycle blocks here
+			result = dialog.run();  // the main cycle blocks here
+#endif
 
 			// Handle the response
 			switch (result) {
 				case Gtk::RESPONSE_ACCEPT:
 				{
-					last_dir = dialog.get_current_folder();  // safe for the future
-
-					std::string file = dialog.get_filename();
+					std::string file;
+#if GTK_CHECK_VERSION(3, 20, 0)
+					file = app_ustring_from_gchar(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog.get())));
+					last_dir = hz::path_get_dirname(file);
+#else
+					file = dialog.get_filename();  // in fs encoding
+					last_dir = dialog.get_current_folder();  // save for the future
+#endif
 					hz::File f(file);
 					if (!f.put_contents(this->contents_)) {  // this will send to debug_ too.
 						gui_show_error_dialog("Cannot save data to file", f.get_error_utf8(), this);
@@ -182,16 +207,18 @@ class GscTextWindow : public AppUIResWidget<GscTextWindow<InstanceSwitch>, Insta
 		}
 
 
+		/// Button click callback
 		void on_close_window_button_clicked()
 		{
-			destroy(this);
+			this->destroy(this);
 		}
 
 
+	private:
 
-		Glib::ustring default_title_;
-		Glib::ustring contents_;
-		std::string save_filename_;
+		Glib::ustring default_title_;  ///< Window title
+		Glib::ustring contents_;  ///< The text to display
+		std::string save_filename_;  ///< Default filename for Save As
 
 
 };
@@ -202,3 +229,5 @@ class GscTextWindow : public AppUIResWidget<GscTextWindow<InstanceSwitch>, Insta
 
 
 #endif
+
+/// @}
